@@ -1,4 +1,5 @@
 use unmp::id::Id;
+use unmp::link::Link;
 use unmp::Connection;
 
 /// etp地址
@@ -22,6 +23,9 @@ impl Addr {
     pub fn id(&self) -> &Id {
         self.conn.id()
     }
+    pub fn link(&self) -> Option<&Link> {
+        self.conn.link()
+    }
     pub fn port(&self) -> u8 {
         self.port
     }
