@@ -4,6 +4,7 @@ use crate::consts::*;
 use crate::datagram::Datagram;
 use crate::sender;
 use crate::Addr;
+use core::sync::atomic::{AtomicU64, AtomicU8, Ordering};
 use fixed_queue::LinearMap;
 use futures_intrusive::buffer::ArrayBuf;
 use futures_intrusive::channel::shared::*;
@@ -20,6 +21,38 @@ type RingBody = ArrayBuf<Respose, [Respose; 1]>;
 type WaitDataSender = GenericSender<Mutex<()>, Respose, RingBody>;
 type WaitDataRecver = GenericReceiver<Mutex<()>, Respose, RingBody>;
 
+struct Timeout {
+    time: AtomicU64,
+    num: AtomicU8,
+}
+impl Timeout {
+    pub fn new(timeout: u64) -> Timeout {
+        Timeout {
+            time: AtomicU64::new(timeout),
+            num: AtomicU8::new(1),
+        }
+    }
+    pub fn get(&self) -> u64 {
+        self.time.load(Ordering::Relaxed)
+    }
+    pub fn update(&self, consum: u64) {
+        let _err = self
+            .num
+            .fetch_update(Ordering::Relaxed, Ordering::Relaxed, |num| {
+                if num < 200 {
+                    let _err =
+                        self.time
+                            .fetch_update(Ordering::Relaxed, Ordering::Relaxed, |time| {
+                                Some((time * num as u64 + 2 * (consum + 100)) / (num as u64 + 1))
+                            });
+                    Some(num + 1)
+                } else {
+                    None
+                }
+            });
+    }
+}
+
 pub struct Config {
     pub retry: u8,
 }
@@ -27,7 +60,7 @@ pub struct Socket {
     port: u8,
     remote: Option<Addr>,
     recver: WaitDataRecver,
-    timeout: (u64, u8),
+    timeout: Timeout,
     cfg: Config,
 }
 impl Socket {
@@ -47,7 +80,7 @@ impl Socket {
             port: port,
             remote: None,
             recver: recver,
-            timeout: (10_000, 1),
+            timeout: Timeout::new(10_000),
             cfg: config,
         })
     }
@@ -56,23 +89,23 @@ impl Socket {
         self.remote = Some(remote)
     }
     /// 发送数据给绑定设备
-    pub async fn send(&mut self, data: &[u8]) -> Result<(), ()> {
+    pub async fn send(&self, data: &[u8]) -> Result<(), ()> {
         if let Some(remote) = &self.remote {
             if let Ok(consum) =
-                sender::send(self.port, remote, data, self.cfg.retry, self.timeout.0).await
+                sender::send(self.port, remote, data, self.cfg.retry, self.timeout.get()).await
             {
-                self.update_timeout(consum);
+                self.timeout.update(consum);
                 return Ok(());
             }
         }
         return Err(());
     }
     /// 发送数据给目标设备
-    pub async fn send_to(&mut self, data: &[u8], remote: &Addr) -> Result<(), ()> {
+    pub async fn send_to(&self, data: &[u8], remote: &Addr) -> Result<(), ()> {
         if let Ok(consum) =
-            sender::send(self.port, remote, data, self.cfg.retry, self.timeout.0).await
+            sender::send(self.port, remote, data, self.cfg.retry, self.timeout.get()).await
         {
-            self.update_timeout(consum);
+            self.timeout.update(consum);
             return Ok(());
         }
         return Err(());
@@ -85,14 +118,6 @@ impl Socket {
             Err(())
         }
     }
-    /// 更新超时时间
-    fn update_timeout(&mut self, consum: u64) {
-        let (timeout, num) = self.timeout;
-        if num < 200 {
-            self.timeout.0 = (timeout * num as u64 + 2 * (consum + 100)) / (num as u64 + 1);
-            self.timeout.1 += 1;
-        }
-    }
 }
 
 pub(crate) fn when_recv(datagram: &Datagram, remote: Addr) {
